// $Id: FreeEnergyModel.h 715 2013-09-17 21:28:50Z ge $
/// \file FreeEnergyModel.h
/// \brief contains the FreeEnergyModel class
///
/// $Revision: 715 $
/// \author Gerald Weber <gweberbh@gmail.com>
#ifndef GBC_EXP_FREEENERGYMODEL_H
#define GBC_EXP_FREEENERGYMODEL_H "$Id: FreeEnergyModel.h 715 2013-09-17 21:28:50Z ge $"
#include "SequenceInfo.h"
#include "SequenceDataset.h"
#include "IndividualNearestNeighbours.h"

namespace gbc_exp
{
template<class _Model=IndividualNearestNeighbours<double> >
class FreeEnergyModel
  {
  public:
    typedef _Model                                 model_type;
    typedef typename model_type::value_type        value_type;
    typedef std::valarray<value_type>              vector_type;
    typedef SequenceInfo<value_type>               sequenceinfo_type;
    typedef SequenceDataset<sequenceinfo_type>     sequence_dataset_type;

    sequence_dataset_type dataset;
    model_type            *model;
    std::string           data_set_type;

    FreeEnergyModel(void) {}


   inline void verify(std::ostream& out)
      {
      dataset.verify(out);
      }

   inline void print_tex(std::ostream& out)
     {
     dataset.print_tex(out);
     }

    /// \brief Calculates the difference between the predicted and the adjusted temperature
    inline void predict_all(void)
      {
      typename sequence_dataset_type::sequence_info_deque_type::iterator si=dataset.Raw_dataset.begin();
      for(si=dataset.Raw_dataset.begin(); si != dataset.Raw_dataset.end(); si++)
        {
        model->Sequence_info=&(*si);
        model->reset_calculation();
        model->melting_temperature();
        }
      }

  };

}
#endif
