// $Id: GibbsMinimization.h 715 2013-09-17 21:28:50Z ge $
/// \file GibbsMinimization.h
/// \brief contains the GibbsMinimization function class
///
/// This is a plugin class for the ParameterSearch function
/// $Revision: 715 $
/// \author Gerald Weber <gweberbh@gmail.com>
#ifndef GBC_EXP_GIBBSMINIMIZATION_H
#define GBC_EXP_GIBBSMINIMIZATION_H "$Id: GibbsMinimization.h 715 2013-09-17 21:28:50Z ge $"
#include "FreeEnergyModel.h"

namespace gbc_exp
{
template<class _NNModel=IndividualNearestNeighbours<double> >
/// \brief This class requires an xml file of data to work
class GibbsMinimization
  {
  public:
    typedef _NNModel                                  model_type;
    typedef typename model_type::value_type           value_type;
    typedef std::valarray<value_type>                 vector_type;
    typedef FreeEnergyModel<model_type>               freeenergy_type;

    freeenergy_type Regression;
    model_type      Calculation;
    bool Debug;

    GibbsMinimization(void): Regression(), Calculation(), Debug(false) 
      {
      Regression.model=&Calculation;
      };

    void read_xml(std::string xmlfile, int restrict_length)
      {
      std::deque<std::string> delete_bp;
      Regression.dataset.read_xml(xmlfile,restrict_length,std::cout);
      }

   inline std::string identification(void) {return std::string("Gibbs Free energy");}

   inline value_type evaluate(void)
     {
     Regression.predict_all();
     Regression.dataset.difference();
     if (Debug) std::cerr << Regression.dataset.Temperature.sqr_diff << " " << std::flush;
     return Regression.dataset.Temperature.sqr_diff;
     }

   inline void print_verification(std::ostream& out)
     {
     Regression.verify(out);
     Regression.dataset.print_temperatures(out);
     }

   inline void print_regression(std::ostream& out)
     {}

   inline void print_tex(std::ostream& out)
     {
     Regression.dataset.difference();
     Regression.print_tex(out);
     }

   inline void randomise_data(void)
     {
     Regression.dataset.randomise_dataset();
     }


   };
};
#endif
